#include "blockchain.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

#include "signature.h"

#define INITIAL_SIGNATURE "FEE1BABEFEE1B16B00B135C000000001"

/* --- init/destroy --- */

chain* chain_init() {
    // hand-building the initial block
    transaction* t = calloc(1, sizeof(transaction));
    t->points      = TOTAL_POINTS;
    t->destination = Administration;
    
    block* b = calloc(1, sizeof(block));
    b->t     = t;
    b->sign  = calloc(sizeof(INITIAL_SIGNATURE) + 1, sizeof(char));
    strcpy(b->sign, INITIAL_SIGNATURE);

    chain* c = calloc(1, sizeof(chain));
    c->head = b;

    return c;
}

void chain_free(chain* c) {
    // TODO HERE
    // free everything
	block* bloc_examine = c->head;
	while (bloc_examine->previous != NULL) {
		block* bloc_suivant = bloc_examine->previous;
		block_free(bloc_examine);
		bloc_examine = bloc_suivant;
	}
	block_free(bloc_examine);
	free(c);
}

/* --- add --- */

void chain_add(chain* c, transaction* t) {
    // TODO HERE
	c->head = block_init(t, c->head);
}

/* --- validate --- */

bool chain_validate(chain* c) {
    // TODO HERE
	block* bloc_examine = c->head;
	while (bloc_examine->previous != NULL) {
		transaction* transaction_examinee = bloc_examine->t;
		char* signature_attendue = signature_sign(bloc_examine->previous->sign, transaction_examinee->source, transaction_examinee->destination, transaction_examinee->points);
		bool correct = (strcmp(bloc_examine->sign, signature_attendue) == 0);
		free(signature_attendue);
		if (!correct) {
			return false;
		}
		bloc_examine = bloc_examine->previous;
	}
	return (strcmp(bloc_examine->sign, INITIAL_SIGNATURE) == 0);
}

/* --- compute points --- */

int* chain_compute(chain* c) {
    // TODO HERE
	int* resultats = calloc(5, sizeof(int));
	block* bloc_examine = c->head;
	while (bloc_examine->previous != NULL) {
		transaction* transaction_examinee = bloc_examine->t;
		resultats[transaction_examinee->destination] += transaction_examinee->points;
		resultats[transaction_examinee->source] -= transaction_examinee->points;
		bloc_examine = bloc_examine->previous;
	}
	transaction* derniere_transaction = bloc_examine->t;
	resultats[derniere_transaction->destination] += derniere_transaction->points;
	return resultats;
}
