#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "blockchain.h"

int main(void) {     
    chain* c = chain_init();

    // check that the signature of the last block is the correct one
    assert(strcmp(c->head->sign, "FEE1BABEFEE1B16B00B135C000000001") == 0);

    transaction* t = transaction_init(20, Administration, Gryffondor);
    chain_add(c, t);

    // check that the signature of the last block is the correct one
    assert(strcmp(c->head->sign, "38e0e5f922071cf49971b5f1450a8676") == 0);

    // check that all block's signatures are valid
    assert(chain_validate(c));

    // check that if we put a false signature the chain doesn't validate
    char* real_sign = c->head->sign;
    c->head->sign = "38e0e5f922071cf49971b5f1450a8675";
    assert(chain_validate(c) == false);
    c->head->sign = real_sign;

    // compute the points of each "maison"
    int* points = chain_compute(c);

    assert(points[Administration] == 4980);
    assert(points[Gryffondor]     == 20);
    assert(points[Poufsouffle]    == 0);
    assert(points[Serdaigle]      == 0);
    assert(points[Serpentard]     == 0);
    free(points);

    chain_add(c, transaction_init(400, Administration, Gryffondor));
    chain_add(c, transaction_init(250, Administration, Poufsouffle));
    chain_add(c, transaction_init(100, Administration, Serpentard));
    chain_add(c, transaction_init(200, Administration, Serdaigle));
    chain_add(c, transaction_init(400, Gryffondor,     Serpentard));
    chain_add(c, transaction_init(180, Administration, Gryffondor));
    chain_add(c, transaction_init(100, Serpentard,     Poufsouffle));
    chain_add(c, transaction_init(100, Serpentard,     Gryffondor));
    chain_add(c, transaction_init(450, Administration, Gryffondor));
    chain_add(c, transaction_init(50,  Administration, Serpentard));

    assert(strcmp(c->head->sign, "a8c31749ffcbd4efeefd4ea00b4885f8") == 0);
    assert(chain_validate(c));

    // IF YOU HAVE FINISHED EVERYTHING ELSE
    // compute and display :
    // - the points of each house
    // - the total of all points
    // - the winning house (without "Administration")

	points = chain_compute(c);
	char noms[5][16] = {"Administration", "Gryffondor", "Poufsouffle", "Serdaigle", "Serpentard"};
	int pt_max = 0;
	int plus_haut = 0;
	for (int i = 1; i < 5; i++) {
		printf("Pour %s : %i points !\n", noms[i], points[i]);
		if (points[i] > pt_max) {
			pt_max = points[i];
			plus_haut = i;
		}
	}
	printf("Et c'est donc une victoire pour %s !\n", noms[plus_haut]);
	free(points);


    chain_free(c);

    printf("All went well\n");
}
